;;; navi2ch-article.el --- article view module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'cl)
(require 'navi2ch-message)
(require 'browse-url)

(defvar navi2ch-article-enable-diff t
  "$B:9J,$r<h$C$F$/$k$+$I$&$+!#(Bnil $B$J$i>o$K:9J,$r<h$C$F$3$J$$(B")

(defvar navi2ch-article-enable-fill nil
  "fill-region $B$9$k$+$I$&$+(B")
(defvar navi2ch-article-enable-fill-list nil
  "fill-region $B$9$k(B $BHD$N%j%9%H(B")
(defvar navi2ch-article-disable-fill-list nil
  "fill-region $B$7$J$$HD$N%j%9%H(B")

(defvar navi2ch-article-link-face 'bold)
(defvar navi2ch-article-link-mouse-face 'highlight)
(defvar navi2ch-article-url-face navi2ch-article-link-face)
(defvar navi2ch-article-url-mouse-face navi2ch-article-link-mouse-face)

(defvar navi2ch-article-parse-field-list
  '(data name)
  "parse $B$9$k%U%#!<%k%I$N%j%9%H!#(B
$BCY$/$F$b$$$$$s$J$i(B '(data mail name) $B$H$+$9$k$H$$$$$+$b(B")

(defvar navi2ch-article-goto-number-recenter t
  "goto-number $B$7$?$H$-$K(B recenter $B$9$k$+$I$&$+(B")

(defvar navi2ch-article-view-range nil
  "$BI=<($9$k%9%l%C%I$NHO0O!#(B
$B=q<0$O(B '(first . last) $B$G!"(B
first $B$,:G=i$+$i$$$/$DI=<($9$k$+!"(B
last $B$,:G8e$+$i$$$/$DI=<($9$k$+!#(B
$BNc$($P!"(B(10 . 50) $B$G!":G=i$N(B10$B$H:G8e$N(B50$B$rI=<((B")

(defvar navi2ch-article-view-range-list
  '((1 . 50)
    (50 . 50)
    (1 . 100)
    (100 . 100))
  "$BI=<($9$k%9%l%C%I$NHO0O$rA*Br$9$k$H$-$K;H$&%j%9%H(B")
  
(defvar navi2ch-article-header-format-function
  'navi2ch-article-default-header-format-function
  "NUMBER NAME MAIL DATE $B$r0z?t$K<h$j!"%l%9$N%X%C%@$rJV$94X?t(B")

(defvar navi2ch-article-mode-map nil)
(unless navi2ch-article-mode-map
  (setq navi2ch-article-mode-map (make-sparse-keymap))
  (define-key navi2ch-article-mode-map "q" 'navi2ch-article-exit)
  (define-key navi2ch-article-mode-map " " 'navi2ch-article-scroll-up)
  (define-key navi2ch-article-mode-map "s" 'navi2ch-article-sync)
  (define-key navi2ch-article-mode-map "S" 'navi2ch-article-sync-disable-diff)
  (define-key navi2ch-article-mode-map "r" 'navi2ch-article-redraw-range)
  (define-key navi2ch-article-mode-map [del] 'navi2ch-article-scroll-down)
  (define-key navi2ch-article-mode-map [backspace] 'navi2ch-article-scroll-down)
  (define-key navi2ch-article-mode-map "\177" 'navi2ch-article-scroll-down)
  ;; (define-key navi2ch-article-mode-map "a" 'navi2ch-article-add-number)
  ;; (define-key navi2ch-article-mode-map "b" 'navi2ch-article-open-number)
  (define-key navi2ch-article-mode-map "w" 'navi2ch-article-write-message)
  (define-key navi2ch-article-mode-map "t" 'navi2ch-toggle-offline)
  (define-key navi2ch-article-mode-map "\r" 'navi2ch-article-select-current-link)
  (define-key navi2ch-article-mode-map [mouse-2] 'navi2ch-article-mouse-select)
  (define-key navi2ch-article-mode-map "g" 'navi2ch-article-goto-number)
  (define-key navi2ch-article-mode-map "l" 'navi2ch-article-pop-point)
  (define-key navi2ch-article-mode-map "m" 'navi2ch-article-push-point)
  (define-key navi2ch-article-mode-map "v" 'navi2ch-article-browse-url)
  (define-key navi2ch-article-mode-map "V" 'navi2ch-article-browse-url-at-point)
  
  (define-key navi2ch-article-mode-map "." 'navi2ch-article-redisplay-current-message)
  (define-key navi2ch-article-mode-map "p" 'navi2ch-article-previous-message)
  (define-key navi2ch-article-mode-map "n" 'navi2ch-article-next-message)
  (define-key navi2ch-article-mode-map [(shift tab)] 'navi2ch-article-previous-link)
  (define-key navi2ch-article-mode-map "\e\C-i" 'navi2ch-article-previous-link)
  (define-key navi2ch-article-mode-map "\C-\i" 'navi2ch-article-next-link)
  (define-key navi2ch-article-mode-map ">" 'navi2ch-article-goto-last-message)
  (define-key navi2ch-article-mode-map "<" 'navi2ch-article-goto-first-message)
  (define-key navi2ch-article-mode-map "\eu" 'navi2ch-article-uudecode-message)
  (define-key navi2ch-article-mode-map "f" 'navi2ch-article-forward-buffer)
  (define-key navi2ch-article-mode-map "b" 'navi2ch-article-backward-buffer)
  (define-key navi2ch-article-mode-map "3" 'navi2ch-three-pain))

(defvar navi2ch-article-buffer-name-prefix "*navi2ch article ")
(defvar navi2ch-article-current-article nil)
(defvar navi2ch-article-current-board nil)
(defvar navi2ch-article-message-list nil)
(defvar navi2ch-article-point-stack nil "$B0LCV$r3P$($H$/(B stack")
(defun navi2ch-article-scroll-up ()
  (interactive)
  (scroll-up)
  (force-mode-line-update t))

(defun navi2ch-article-scroll-down ()
  (interactive)
  (scroll-down)
  (force-mode-line-update t))

(defun navi2ch-article-get-url (board article)
  (concat (cdr (assq 'uri board))
          "dat/" (cdr (assq 'artid article))
          ".dat"))

(defun navi2ch-article-get-file-name (board article)
  (navi2ch-expand-file-name
   (concat (cdr (assq 'id board))
           "/" (cdr (assq 'artid article))
           ".dat")))

(defun navi2ch-article-get-info-file-name (board article)
  (navi2ch-expand-file-name
   (concat (cdr (assq 'id board))
           "/info/" (cdr (assq 'artid article)))))

(defun navi2ch-article-inside-range-p (num range len)
  "NUM $B$,(B RANGE $B$G<($9HO0O$KF~$C$F$k$+(B
LEN $B$O(B RANGE $B$GHO0O$r;XDj$5$l$k(B list $B$ND9$5(B"
  (or (not range)
      (<= num (car range))
      (> num (- len (cdr range)))))

(defun navi2ch-article-parse-message (list range)
  "LIST $B$r(B RANGE $B$NHO0O$G(B parse $B$9$k!#(B
$B$9$G$K(B parse $B$5$l$F$kHO0O$O(B parse $B$7$J$$(B"
  (let ((first (car range))
        (last (cdr range))
        message-list)
    (message "parsing current messages...")
    (setq message-list
          (mapcar
           (lambda (x)
             (if (and (stringp (cdr x))
                      (navi2ch-article-inside-range-p
                       (car x) range (length list)))
                 (let ((strs (split-string (cdr x) "\\(,\\|<> *\\)"))
                       list2)
                   (setq list2
                         (list
                          (cons 'name (nth 0 strs))
                          (cons 'mail (nth 1 strs))
                          (cons 'date (nth 2 strs))
                          (cons 'data (nth 3 strs))))
                   ;; (cons 'title (nth 4 strs))

                   ;; $B%?%0$=$NB>$NCV49(B
                   (let ((case-fold-search nil)
                         str)
                     (setq str (cdr (assq 'data list2)))
                     (let (start)
                       (while (setq start (string-match " <br> " str start))
                         (setq str (replace-match "\n" nil nil str))))
                     (setcdr (assq 'data list2) str)
                     (dolist (elt navi2ch-article-parse-field-list)
                       (setq str (cdr (assq elt list2)))
                       (setq str (navi2ch-replace-html-tag str))
                       (setcdr (assq elt list2) str)))
                   (cons (car x) list2))
               x))
           list))
    (message "parsing current messages...done")
    message-list))

(defun navi2ch-article-get-message-list (file &optional begin end)
  "FILE $B$N(B BEGIN $B$+$i(B END $B$^$G$NHO0O$+$i%9%l$N(B list $B$r:n$k(B"
  (when (file-exists-p file)
    (let ((content (with-temp-buffer
                     (navi2ch-insert-file-contents file begin end)
                     (buffer-string)))
          (i 1)
          message-list)
      (message "splitting current messages...")
      (setq message-list
            (mapcar
             (lambda (x)
               (let (elt)
                 (setq elt (cons i x))
                 (setq i (1+ i))
                 elt))
             (split-string content "\n")))
      (message "splitting current messages...done")
      message-list)))
                      
(defun navi2ch-article-append-message-list (list1 list2)
  (let ((num (length list1))
        list)
    (message "appending current messages...")
    (setq list (append list1
                       (mapcar
                        (lambda (x)
                          (setq num (1+ num))
                          (cons num (cdr x)))
                        list2)))
    (message "appending current messages...done")
    list))

(defun navi2ch-article-insert-messages (list range)
  "LIST $B$r@07A$7$FA^F~$9$k(B"
  (message "inserting current messages...")
  (let ((len (length navi2ch-article-message-list)))
    (dolist (elt list)
      (when (navi2ch-article-inside-range-p (car elt) range len)
        (let ((num (car elt))
              (elt (cdr elt))
              (begin (point-marker)))
          (if (assq 'point elt)
              (setcdr (assq 'point elt) (point-marker))
            (nconc elt (list (cons 'point (point-marker)))))
          (let ((p (point)))
            (insert (funcall navi2ch-article-header-format-function
                             num
                             (cdr (assq 'name elt))
                             (cdr (assq 'mail elt))
                             (cdr (assq 'date elt))))
            (put-text-property p (1+ p) 'current-number num))
          (let ((p (point)))
            (insert (cdr (assq 'data elt)) "\n")
            (navi2ch-article-set-link-property p (point))
            (let ((id (cdr (assq 'id navi2ch-article-current-board))))
              (when (or (member id navi2ch-article-enable-fill-list)
                        (and (not (member id navi2ch-article-disable-fill-list))
                             navi2ch-article-enable-fill))
                (set-hard-newline-properties p (point))
                (let ((fill-column (- (window-width) 5))
                      (use-hard-newlines t))
                  (fill-region p (point))))))
          (let ((p (point)))
            (insert "\n")
            (put-text-property p (point) 'face 'underline))
          (insert "\n")))))
    (message "inserting current messages...done"))

(defun navi2ch-article-default-header-format-function (number name mail date)
  "$B%G%U%)%k%H$N%X%C%@$r%U%)!<%^%C%H$9$k4X?t(B
$B%X%C%@$N(Bface $B$rIU$1$k$N$b$3$3$G!#(B"
  (let (str p)
    (setq str
          (concat "From: [" (number-to-string number) "] "
                  name
                  (if (string= mail "")
                      ""
                    (concat " <" mail ">"))
                  "\n")
          p (length str))
    (put-text-property 0 (1- p) 'face 'underline str)
    (setq str (concat str "Date: " date "\n\n"))
    (put-text-property p (- (length str) 2) 'face 'underline str)
    str))

(defun navi2ch-article-set-link-property (beg end)
  ">>1 $B$H$+(B http:// $B$K(B property $B$rIU$1$k(B"
  (save-excursion
    (save-restriction
      (narrow-to-region beg end)
      (goto-char (point-min))
      (while (re-search-forward
              "[>$B!d(B][>$B!d(B]?\\([0-9$B#0#1#2#3#4#5#6#7#8#9(B]+\\)" nil t)
        (add-text-properties
         (match-beginning 0)
         (match-end 0)
         (list 'face navi2ch-article-link-face
               'link t
               'mouse-face navi2ch-article-link-mouse-face
               'number (string-to-number
                        (japanese-hankaku (match-string 1))))))
      (goto-char (point-min))
      (while (re-search-forward
              "h?ttp://\\([-a-zA-Z0-9_=?#$@~`%&*+|\\/.,:]+\\)" nil t)
        (add-text-properties
         (match-beginning 0)
         (match-end 0)
         (list 'face navi2ch-article-link-face
               'link t
               'mouse-face navi2ch-article-link-mouse-face
               'url (concat "http://" (match-string 1))))))))
  
(defun navi2ch-article-view-article (board article &optional force)
  "$B%9%l$r8+$k!#(Bforce $B$G6/@)FI$_9~$_(B"
  (let ((buf-name (navi2ch-article-get-buffer-name board article)))
    (if (get-buffer buf-name)
        (progn
          (switch-to-buffer buf-name)
          (navi2ch-article-sync force))
      (switch-to-buffer (get-buffer-create buf-name))
      (navi2ch-article-mode)
      (setq navi2ch-article-current-board board
            navi2ch-article-current-article article)
      (navi2ch-article-sync force t))))

(defun navi2ch-article-mode ()
"\\{navi2ch-article-mode-map}"
  (interactive)
  (setq major-mode 'navi2ch-article-mode)
  (setq mode-name "Navi2ch Article")
  (setq buffer-read-only t)
  (make-local-variable 'navi2ch-article-current-article)
  (make-local-variable 'navi2ch-article-current-board)
  (make-local-variable 'navi2ch-article-message-list)
  (make-local-variable 'navi2ch-article-point-stack)
  (make-local-variable 'truncate-partial-width-windows)
  (setq truncate-partial-width-windows nil)
  (use-local-map navi2ch-article-mode-map)
  (setq navi2ch-article-point-stack nil)
  (run-hooks 'navi2ch-article-mode-hook))

(defun navi2ch-article-exit ()
  (interactive)
  ;; (navi2ch-article-add-number)
  (delete-windows-on (current-buffer))
;;  (bury-buffer navi2ch-article-buffer-name)
  (when (get-buffer navi2ch-board-buffer-name)
    (let ((win (get-buffer-window navi2ch-board-buffer-name)))
      (if win
          (select-window win)
        (switch-to-buffer navi2ch-board-buffer-name)))))

(defun navi2ch-article-sync (&optional force first)
  "$B%9%l$r99?7$9$k!#(Bforce $B$J$i6/@)!#(B
first $B$,(B nil $B$J$i$P!"%U%!%$%k$,99?7$5$l$F$J$1$l$P2?$b$7$J$$(B"
  (interactive "P")
  (let* ((list navi2ch-article-message-list)
         (range navi2ch-article-view-range)
         (article navi2ch-article-current-article)
         (board navi2ch-article-current-board)
         (navi2ch-net-force-update (or navi2ch-net-force-update
                                       force))
         (url (navi2ch-article-get-url board article))
         (file (navi2ch-article-get-file-name board article))
         (old-size (nth 7 (file-attributes file)))
         updated)
    
    (unless navi2ch-offline
      (setq updated (navi2ch-net-update-file
                     url file navi2ch-article-enable-diff)))
    (cond
     (first
      (setq list (navi2ch-article-get-message-list file)))
     (updated
      (navi2ch-article-add-number)
      (setq list (navi2ch-article-append-message-list
                  list
                  (navi2ch-article-get-message-list file old-size)))))
    (let ((buffer-read-only nil))
      (erase-buffer)
      (setq list (navi2ch-article-parse-message list range)
            navi2ch-article-message-list list)
      (navi2ch-article-insert-messages list range))
      (navi2ch-article-open-number)
      (setq mode-line-buffer-identification
            (concat (cdr (assq 'subject article))
                    " (" (cdr (assq 'response article))
                    "/" (length list) ")"))
      (force-mode-line-update t)
      list))

(defun navi2ch-article-sync-disable-diff (&optional force)
  (interactive "P")
  (let ((navi2ch-article-enable-diff nil))
    (navi2ch-article-sync force)))

(defun navi2ch-article-redraw ()
  "$B8=:_I=<($7$F$k%9%l$rI=<($7$J$*$9(B"
  (let ((buffer-read-only nil)
        (p (point)))
    (navi2ch-article-add-number)
    (erase-buffer)
    (setq navi2ch-article-message-list
          (navi2ch-article-parse-message
           navi2ch-article-message-list
           navi2ch-article-view-range))
    (navi2ch-article-insert-messages navi2ch-article-message-list
                                     navi2ch-article-view-range)
    (navi2ch-article-open-number)
    (goto-char p)))

(defun navi2ch-article-select-view-range ()
  "$BI=<($9$kHO0O$r%-!<%\!<%I%a%K%e!<$GA*Br$9$k(B"
  (interactive)
  (save-window-excursion
    (delete-other-windows)
    (let ((buf (get-buffer-create "*select view range*")))
      (save-excursion
        (set-buffer buf)
        (erase-buffer)
        (insert (format "   %8s %8s\n" "first" "last"))
        (insert (format "0: %17s\n" "all range"))
        (let ((i 1))
          (dolist (x navi2ch-article-view-range-list)
            (insert (format "%d: %8d %8d\n" i (car x) (cdr x)))
            (setq i (1+ i)))))
      (display-buffer buf)
      (let (n)
        (setq n (read-char "input: "))
        (when (or (< n ?0) (> n ?9))
          (error "%c is bad key" n))
        (setq n (- n ?0))
        (setq navi2ch-article-view-range
              (if (eq n 0) nil
                (nth (1- n) navi2ch-article-view-range-list))))
      (kill-buffer buf))))

(defun navi2ch-article-redraw-range ()
  "$BI=<($9$kHO0O$r;XDj$7$?8e(B redraw"
  (interactive)
  (navi2ch-article-select-view-range)
  (navi2ch-article-redraw))

(defun navi2ch-article-add-number ()
  (interactive)
  (let ((elt (assq 'number navi2ch-article-current-article)))
    (if elt
        (setcdr elt (navi2ch-article-get-current-number))
      (setcdr navi2ch-article-current-article
              (cons (cons 'number (navi2ch-article-get-current-number))
                    (cdr navi2ch-article-current-article))))))

(defun navi2ch-article-open-number ()
  (interactive)
  (let ((elt (cdr (assq 'number navi2ch-article-current-article))))
    (navi2ch-article-goto-number (or elt 1))))

(defun navi2ch-article-write-message ()
  (interactive)
  (navi2ch-message-write-message navi2ch-board-current-board
                                 navi2ch-article-current-article))

(defun navi2ch-article-select-current-link ()
  (interactive)
  (let ((prop (get-text-property (point) 'number)))
    (if prop
        (navi2ch-article-goto-number prop t)
      (setq prop (get-text-property (point) 'url))
      (if (navi2ch-2ch-url-p prop)
          (navi2ch-article-view-article
           (navi2ch-url-to-board prop)
           (navi2ch-url-to-article prop))
        (browse-url prop)))))

(defun navi2ch-article-mouse-select (e)
  (interactive "e")
  (mouse-set-point e)
  (navi2ch-article-select-current-link))

(defun navi2ch-article-recenter (num)
  "NUM $BHVL\$N%l%9$r2hLL$N0lHV>e$K(B"
  (save-excursion
    (goto-char (cdr (assq 'point (navi2ch-article-get-message num))))
    (recenter 0)))

(defun navi2ch-article-goto-number (num &optional save)
  "NUM $BHVL\$N%l%9$K0\F0(B"
  (interactive "ninput number: ")
  (when (or (interactive-p) save)
    (navi2ch-article-push-point))
  (let ((len (length navi2ch-article-message-list))
        (range navi2ch-article-view-range))
    (unless (navi2ch-article-inside-range-p num range len)
      (setq num (1+ (- len (cdr range)))))
    (goto-char (cdr (assq 'point (navi2ch-article-get-message num)))))
  (if navi2ch-article-goto-number-recenter
      (navi2ch-article-recenter num))
  (force-mode-line-update t))

(defun navi2ch-article-pop-point ()
  "stack $B$+$i(B pop $B$7$?0LCV$K0\F0$9$k(B"
  (interactive)
  (let ((point (pop navi2ch-article-point-stack)))
    (if point
        (goto-char point)
      (message "stack is empty"))))

(defun navi2ch-article-push-point (&optional point)
  "$B8=:_0LCV$+(B POINT $B$r(B stack $B$K(B push $B$9$k(B"
  (interactive)
  (push (or point (point)) navi2ch-article-point-stack)
  (message "push current point"))

(defun navi2ch-article-goto-last-message ()
  "$B:G8e$N%l%9$X(B"
  (interactive)
  (navi2ch-article-goto-number
   (save-excursion
     (goto-char (point-max))
     (navi2ch-article-get-current-number))) t)

(defun navi2ch-article-goto-first-message ()
  "$B:G=i$N%l%9$X(B"
  (interactive)
  (navi2ch-article-goto-number
   (save-excursion
     (goto-char (point-min))
     (navi2ch-article-get-current-number))) t)

(defun navi2ch-article-get-message (num)
  "NUM $BHVL\$N%l%9$rF@$k(B"
  (cdr (assq num navi2ch-article-message-list)))

(defun navi2ch-article-get-current-number ()
  "$B:#$N0LCV$N%l%9$NHV9f$rF@$k(B"
  (or (get-text-property (point) 'current-number)
      (let ((pos (previous-single-property-change
                  (point)
                  'current-number)))
        (or (get-text-property pos 'current-number)
            (get-text-property (1- pos) 'current-number)))))

(defun navi2ch-article-browse-url (&optional board article start end nofirst)
  "$B%V%i%&%6$G!"%9%l$r8+$k!#(BSTART, END, NOFIRST $B$GHO0O$r;XDj$9$k(B"
  (interactive)
  (or board (setq board navi2ch-article-current-board))
  (or article (setq article navi2ch-article-current-article))
  (let ((url (cdr (assq 'uri board))))
    (setq url (concat
               (navi2ch-replace-string "\\(http://[^/]+/\\).+"
                                       "\\1" url)
               "test/read.cgi?bbs="
               (cdr (assq 'id board))
               "&key="
               (cdr (assq 'artid article))
               (when start
                 (format "&st=%d" start))
               (when end
                 (format "&to=%d" end))
               (when nofirst
                 "&nofirst=true")))
    (browse-url url)))
                            
(defun navi2ch-article-browse-url-at-point ()
  "$B:#$$$k%l%9$r%V%i%&%6$G8+$k(B"
  (interactive)
  (navi2ch-article-browse-url nil nil
                              (navi2ch-article-get-current-number)
                              (navi2ch-article-get-current-number)
                              t))

(defun navi2ch-article-redisplay-current-message ()
  "$B:#$$$k%l%9$r2hLL$NCf?4(B($B>e!)(B)$B$K(B"
  (interactive)
  (navi2ch-article-recenter
   (navi2ch-article-get-current-number)))

(defun navi2ch-article-next-message ()
  "$B<!$N%a%C%;!<%8$X(B"
  (interactive)
  (let ((num (1+ (navi2ch-article-get-current-number)))
        (len (length navi2ch-article-message-list))
        (range navi2ch-article-view-range))
    (navi2ch-article-goto-number
     (if (navi2ch-article-inside-range-p num range len)
         num
       (1+ (- len (cdr range)))))))

(defun navi2ch-article-previous-message ()
  "$BA0$N%a%C%;!<%8$X(B"
  (interactive)
  (let ((num (1- (navi2ch-article-get-current-number)))
        (len (length navi2ch-article-message-list))
        (range navi2ch-article-view-range))
    (navi2ch-article-goto-number
     (if (navi2ch-article-inside-range-p num range len)
         num
       (car range)))))

(defun navi2ch-article-next-link ()
  "$B<!$N%j%s%/$X(B"
  (interactive)
  (let ((point (or (and (get-text-property (point) 'link)
                        (next-single-property-change (point) 'link))
                   (point))))
    (setq point (next-single-property-change point 'link))
    (and point (goto-char point))))

(defun navi2ch-article-previous-link ()
  "$BA0$N%j%s%/$X(B"
  (interactive)
  (let ((point (or (and (get-text-property (point) 'link)
                        (previous-single-property-change (point) 'link))
                   (point))))
    (setq point (previous-single-property-change point 'link))
    (or (get-text-property point 'link)
        (setq point (previous-single-property-change point 'link)))
    (and point (goto-char point))))

(defun navi2ch-article-uudecode-message ()
  (interactive)
  (with-temp-buffer
    (insert (cdr
             (assq 'data
                   (save-excursion
                     (set-buffer (navi2ch-article-current-buffer))
                     (navi2ch-article-get-message
                      (navi2ch-article-get-current-number))))))
    (navi2ch-uudecode-region (point-min) (point-max))))
  
(defun navi2ch-article-get-buffer-name (board article)
  (concat navi2ch-article-buffer-name-prefix
          (cdr (assq 'id board))
          "/"
          (cdr (assq 'artid article))))
                    
(defun navi2ch-article-buffer-list ()
  "`navi2ch-article-mode' $B$N(B buffer $B$N(B list $B$rJV$9(B"
  (let (list)
    (dolist (x (buffer-list))
      (when (save-excursion
              (set-buffer x)
              (eq major-mode 'navi2ch-article-mode))
        (setq list (cons x list))))
    (nreverse list)))

(defun navi2ch-article-current-buffer ()
  "`buffer-list' $B$N0lHV:G=i$N(B `navi2ch-article-mode' $B$N(B buffer $B$rJV$9(B"
  (let ((list (buffer-list)))
    (catch 'loop
      (while list
        (when (save-excursion
                (set-buffer (car list))
                (eq major-mode 'navi2ch-article-mode))
          (throw 'loop (car list)))
        (setq list (cdr list)))
      nil)))

(defun navi2ch-article-forward-buffer ()
  "$B<!$N(B article buffer $B$X(B"
  (interactive)
  (let (buf)
    (dolist (x (buffer-list))
      (when (save-excursion
              (set-buffer x)
              (eq major-mode 'navi2ch-article-mode))
        (setq buf x)))
    (switch-to-buffer buf)))
  
(defun navi2ch-article-backward-buffer ()
  "$BA0$N(B article buffer $B$X(B"
  (interactive)
  (bury-buffer)
  (switch-to-buffer (navi2ch-article-current-buffer)))

(provide 'navi2ch-article)

;;; navi2ch-article.el ends here