;;; navi2ch-list.el --- bord list module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:

(require 'navi2ch-board)
(require 'cl)

(defvar navi2ch-list-buffer-name "*navi2ch list*")
(defvar navi2ch-list-window-width 20)
(defvar navi2ch-list-bookmark-mode nil)
(defvar navi2ch-list-board-name-list nil)

(defvar navi2ch-list-mode-map nil)
(unless navi2ch-list-mode-map
  (setq navi2ch-list-mode-map (make-sparse-keymap))
  (define-key navi2ch-list-mode-map "\r" 'navi2ch-list-select-current-board)
  (define-key navi2ch-list-mode-map "q" 'navi2ch-list-exit)
  (define-key navi2ch-list-mode-map "s" 'navi2ch-list-sync)
  (define-key navi2ch-list-mode-map " " 'navi2ch-list-select-current-board)
  (define-key navi2ch-list-mode-map [del] 'scroll-down)
  (define-key navi2ch-list-mode-map [backspace] 'scroll-down)
  (define-key navi2ch-list-mode-map "n" 'next-line)
  (define-key navi2ch-list-mode-map "p" 'previous-line)
  (define-key navi2ch-list-mode-map "a" 'navi2ch-list-add-bookmark)
  (define-key navi2ch-list-mode-map "d" 'navi2ch-list-delete-bookmark)
  (define-key navi2ch-list-mode-map "b" 'navi2ch-list-toggle-bookmark)
  (define-key navi2ch-list-mode-map "t" 'navi2ch-toggle-offline)
  (define-key navi2ch-list-mode-map [mouse-2] 'navi2ch-list-mouse-select)
  (define-key navi2ch-list-mode-map "g" 'navi2ch-list-goto-board)
  (define-key navi2ch-list-mode-map ">" 'end-of-buffer)
  (define-key navi2ch-list-mode-map "<" 'beginning-of-buffer)
  (define-key navi2ch-list-mode-map "3" 'navi2ch-three-pain))

(defsubst navi2ch-list-get-file-name (&optional name)
  (navi2ch-expand-file-name
   (or name "board.txt")))

(defun navi2ch-list-get-board-name-list (file)
  "file $B$+$i(B $BHD$N(B list $B$rF@$k(B"
  (when (file-exists-p file)
    (with-temp-buffer
      (navi2ch-insert-file-contents file)
      (goto-char (point-min))
      (let ((prev (point))
            list)
        (while (not (eobp))
          (forward-line 3)
          (let ((strs (split-string
                       (buffer-substring prev (point)) "\n")))
            (setq list (cons (list (cons 'name (nth 0 strs))
                                   (cons 'uri (nth 1 strs))
                                   (cons 'id (nth 2 strs)))
                             list))
            (setq prev (point))))
        (nreverse list)))))
       
(defun navi2ch-list-insert-board-names (list)
  "list $B$NFbMF$rA^F~(B"
  (let ((prev (point)))
    (dolist (elt list)
      (insert (cdr (assq 'name elt)) "\n")
      (put-text-property prev (point) 'board elt)
      (put-text-property prev (1- (point)) 'mouse-face 'highlight)
      (setq prev (point)))))

(defun navi2ch-list-select-current-board (&optional force)
  "$BHD$rA*$V(B"
  (interactive "P")
  (let ((board (get-text-property (point) 'board)))
    (if board
        (navi2ch-list-select-board board force)
      (message "can't select this line!"))))

(defun navi2ch-list-select-board (board &optional force)
  (delete-windows-on (get-buffer navi2ch-board-buffer-name))
  (dolist (x (navi2ch-article-buffer-list))
    (delete-windows-on x))
  (split-window-horizontally navi2ch-list-window-width)
  (other-window 1)
  (navi2ch-board-select-board board force))
  
  
(defun navi2ch-list-mode ()
  "\\{navi2ch-list-mode-map}"
  (interactive)
  (setq major-mode 'navi2ch-list-mode)
  (setq mode-name "Navi2ch List")
  (setq buffer-read-only t)
  (use-local-map navi2ch-list-mode-map)
  (run-hooks 'navi2ch-list-mode-hook)
  (force-mode-line-update))

(defun navi2ch-list ()
  (interactive)
  (if (get-buffer navi2ch-list-buffer-name)
      (switch-to-buffer navi2ch-list-buffer-name)
    (switch-to-buffer (get-buffer-create navi2ch-list-buffer-name))
    (navi2ch-list-mode)
    (save-excursion
      (navi2ch-list-sync nil t))))

(defun navi2ch-list-exit ()
  (interactive)
  (navi2ch-board-exit)
  (delete-windows-on (get-buffer navi2ch-list-buffer-name))
  (bury-buffer navi2ch-list-buffer-name))
  
(defun navi2ch-list-sync (&optional force first)
  (interactive "P")
  (setq navi2ch-list-bookmark-mode nil)
  (save-excursion
    (let ((buffer-read-only nil)
          (navi2ch-net-force-update (or navi2ch-net-force-update
                                        force))
          (file (navi2ch-list-get-file-name))
          updated)
      (unless navi2ch-offline
        (setq updated (navi2ch-net-update-file
                       navi2ch-remote-board-file-url file)))
      (when (or first updated)
        (erase-buffer)
        (setq navi2ch-list-board-name-list
              (navi2ch-list-get-board-name-list file))
        (navi2ch-list-insert-board-names navi2ch-list-board-name-list)))))

(defun navi2ch-list-add-bookmark ()
  (interactive)
  (unless navi2ch-list-bookmark-mode
    (let ((board (get-text-property (point) 'board)))
      (if board
	  (let ((name (cdr (assq 'name board)))
		(uri (cdr (assq 'uri board)))
		(id (cdr (assq 'id board)))
		(file-name (navi2ch-list-get-file-name "bookmark.txt"))
                (coding-system-for-write navi2ch-net-coding-system)
                (coding-system-for-read navi2ch-net-coding-system))
            (with-temp-file file-name
              (when (file-exists-p file-name)
                (insert-file-contents file-name)
                (goto-char (point-max)))
              (insert name "\n" uri "\n" id "\n")))
        (message "can't select this line!")))))

(defun navi2ch-list-delete-bookmark ()
  (interactive)
  (if (and navi2ch-list-bookmark-mode
	   (y-or-n-p "Delete this bookmark?"))
      (let ((board (get-text-property (point) 'board)))
        (if board
            (progn
              (setq navi2ch-list-board-name-list
                    (delq board navi2ch-list-board-name-list))
              (let ((coding-system-for-write navi2ch-net-coding-system)
                    (coding-system-for-read navi2ch-net-coding-system))
                (with-temp-file (navi2ch-list-get-file-name "bookmark.txt")
                  (dolist (x navi2ch-list-board-name-list)
                    (dolist (y '(name uri id))
                      (insert (cdr (assq y x)) "\n")))))
              (let ((buffer-read-only nil))
                (delete-region (save-excursion (beginning-of-line) (point))
                               (save-excursion (forward-line) (point)))))
          (message "can't select this line!")))))

(defun navi2ch-list-toggle-bookmark ()
  (interactive)
  (setq navi2ch-list-bookmark-mode (not navi2ch-list-bookmark-mode))
  (let ((buffer-read-only nil))
    (save-excursion
      (erase-buffer)
      (setq navi2ch-list-board-name-list
            (navi2ch-list-get-board-name-list
             (navi2ch-list-get-file-name
              (and navi2ch-list-bookmark-mode
                   (file-exists-p (navi2ch-list-get-file-name "bookmark.txt"))
                   "bookmark.txt"))))
      (navi2ch-list-insert-board-names navi2ch-list-board-name-list))))

(defun navi2ch-list-mouse-select (e)
  (interactive "e")
  (beginning-of-line)
  (mouse-set-point e)
  (save-excursion
    (navi2ch-list-select-current-board)))

(defun navi2ch-list-goto-board ()
  (interactive)
  (let ((alist (mapcar
                (lambda (x)
                  (cons (cdr (assq 'id x)) x))
                navi2ch-list-board-name-list)))
    (navi2ch-list-select-board
     (cdr (assoc (completing-read "board name: " alist nil t) alist)))))

(provide 'navi2ch-list)

;;; navi2ch-list.el ends here