;;; navi2ch.el --- Navigator for 2ch for Emacsen

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'navi2ch-net)
(require 'navi2ch-util)
(require 'navi2ch-list)

(defvar navi2ch-directory "~/.navi2ch")
(defvar navi2ch-remote-board-file-url
  "http://www2.ocn.ne.jp/~luckland/text/board.txt"
  "board.txt $B$N(B url")
(defvar navi2ch-offline nil
  "$B%*%U%i%$%s%b!<%I$+$I$&$+(B")

(defvar navi2ch-version "1.0")

(defun navi2ch ()
  "Navigator for 2ch for Emacs"
  (interactive)
  (navi2ch-list))

(defun navi2ch-toggle-offline ()
  (interactive)
  (message (if navi2ch-offline
               "online"
             "offline"))
  (setq navi2ch-offline (not navi2ch-offline)))

(defun navi2ch-save-info (file info)
  (let ((dir (file-name-directory file)))
    (unless (file-exists-p dir)
      (make-directory dir t)))
  (with-temp-file file
    (let ((standard-output (current-buffer)))
      (prin1 info))))

(defun navi2ch-load-info (file)
  (when (file-exists-p file)
    (with-temp-buffer
      (insert-file-contents file)
      (car (read-from-string (buffer-string))))))

(defun navi2ch-three-pain ()
  (interactive)
  (delete-other-windows)
  (let* ((list-buf (get-buffer navi2ch-list-buffer-name))
         (board-buf (get-buffer navi2ch-board-buffer-name))
         (art-buf (navi2ch-article-current-buffer))
         (buf (when (or (eq (current-buffer) list-buf)
                        (eq (current-buffer) board-buf)
                        (eq (current-buffer) art-buf))
                (setq buf (current-buffer)))))
    (when list-buf
      (switch-to-buffer navi2ch-list-buffer-name)
      (when board-buf
        (split-window-horizontally navi2ch-list-window-width)
        (other-window 1)))
    (when board-buf
      (switch-to-buffer navi2ch-board-buffer-name)
      (when art-buf
        (split-window-vertically navi2ch-board-window-height)
        (other-window 1)))
    (when art-buf
      (switch-to-buffer art-buf))
    (select-window (get-buffer-window buf))))

(provide 'navi2ch)

;;; navi2ch.el ends here